"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const glob_1 = __importDefault(require("glob"));
const node_cron_1 = __importDefault(require("node-cron"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const child_process_1 = __importDefault(require("child_process"));
const zignis_1 = require("zignis");
const debug = zignis_1.Utils.debug('zignis-plugin-zhike:cron');
const spawnSync = child_process_1.default.spawnSync;
const DEFAULT_EXPIRE_MILLISECONDS = 60000;
// 执行器
const shell = {
    // 执行单条系统命令
    exec: function (action, initInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (typeof action === 'string') {
                    debug(`Action: [${action}] executed!`);
                    const parts = action.split(/\s+/g);
                    spawnSync(parts[0], parts.slice(1), {
                        stdio: [process.stdin, process.stdout, process.stderr]
                    });
                }
                else if (zignis_1.Utils._.isArray(action)) {
                    let command = action[0], args;
                    if (zignis_1.Utils._.isArray(action[1])) {
                        args = action[1];
                    }
                    else {
                        args = action.slice(1);
                    }
                    debug(`Action: [${command} ${args.join(' ')}] executed!`);
                    spawnSync(command, args, {
                        stdio: [process.stdin, process.stdout, process.stderr]
                    });
                }
                else if (typeof action === 'function') {
                    const name = action.name ? action.name : 'function';
                    debug(`Action: [${name}] executed!`);
                    yield action(initInfo);
                }
            }
            catch (e) {
                debug(e.stack);
            }
        });
    },
    // 执行多条系统命令
    series: function (actions, initInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            const loops = actions.concat();
            const execNext = function () {
                return __awaiter(this, void 0, void 0, function* () {
                    yield shell.exec(loops.shift(), initInfo);
                    if (loops.length > 0) {
                        yield execNext();
                    }
                });
            };
            yield execNext();
        });
    }
};
exports.command = 'cron [job]';
exports.desc = `zhike cron system`;
exports.builder = function (yargs) { };
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!argv.cronDir || !fs_1.default.existsSync(argv.cronDir)) {
            console.log(zignis_1.Utils.chalk.red('"cronDir" missing in config file or not exist in current directory!'));
            return;
        }
        const config = zignis_1.Utils.getCombinedConfig();
        try {
            // 通过 Hook 进行初始化动作
            const initInfo = yield zignis_1.Utils.invokeHook('zhike_cron');
            // run specific job for testing, ignore disabled property
            if (argv.job) {
                if (fs_1.default.existsSync(path_1.default.resolve(process.cwd(), argv.job))) {
                    const jobModule = require(path_1.default.resolve(process.cwd(), argv.job));
                    if (jobModule && jobModule.actions && zignis_1.Utils._.isArray(jobModule.actions)) {
                        yield shell.series(jobModule.actions, initInfo);
                        process.exit(0);
                    }
                    else {
                        zignis_1.Utils.error('Job not valid');
                    }
                }
                else {
                    zignis_1.Utils.error('Job not found');
                }
            }
            const { redis } = yield zignis_1.Utils.invokeHook('components');
            const redisInstance = yield redis();
            // Redis锁，加锁
            const lock = function (redisKey, redisValue, timeout) {
                return __awaiter(this, void 0, void 0, function* () {
                    return yield redisInstance.eval('return redis.call("set", KEYS[1], ARGV[1], "NX", "PX", ARGV[2])', 1, redisKey, redisValue, timeout);
                });
            };
            // Redis锁，解锁
            const unlock = function (redisKey, redisValue) {
                return __awaiter(this, void 0, void 0, function* () {
                    return yield redisInstance.eval('if redis.call("get", KEYS[1]) == ARGV[1] then return redis.call("del", KEYS[1]) else return 0 end', 1, redisKey, redisValue);
                });
            };
            const jobs = {};
            glob_1.default
                .sync('*.js', {
                cwd: path_1.default.resolve(process.cwd(), config.cronDir)
            })
                .map(function (job) {
                jobs[job] = require(path_1.default.resolve(process.cwd(), config.cronDir, job));
            });
            // 注册计划任务
            if (Object.keys(jobs).length > 0) {
                Object.keys(jobs).forEach(key => {
                    if (jobs[key].disabled)
                        return; // ignore disabled job
                    if (jobs[key].env && jobs[key].env !== zignis_1.Utils.getNodeEnv())
                        return; // ignore if env not match
                    node_cron_1.default.schedule(jobs[key].schedule, function () {
                        const redisKey = `${config.name}:cronjob:${key}`;
                        const redisValue = Math.random();
                        debug(`${new Date().toLocaleString()} - JOB: [${key}] started!`);
                        (() => __awaiter(this, void 0, void 0, function* () {
                            const ok = yield lock(redisKey, redisValue, jobs[key].duration ? jobs[key].duration : DEFAULT_EXPIRE_MILLISECONDS);
                            if (ok) {
                                yield shell.series(jobs[key].actions, initInfo);
                                yield unlock(redisKey, redisValue);
                            }
                            else {
                                debug('lock acquire failed.');
                            }
                        }))();
                    });
                });
            }
            else {
                zignis_1.Utils.error('No enabled cronjob found');
            }
        }
        catch (e) {
            zignis_1.Utils.error(e.stack);
        }
    });
};
